  close all;
  clear;
  
  format compact;
  format short g;

%% Mesh
  fem.geom = rect2;
  fem.mesh = meshinit(fem);
  
%% Set up EIM problem

 % Function
  g = @(x1,x2,mu1,mu2) 1./sqrt((x1-mu1).^2+(x2-mu2).^2);

 % Parameters
  tol = 1e-3;
  n_train = 15;
  n_test = 50;
    
 % Training set (G)
  [mu1,mu2] = meshgrid(-logspace(-2,0,n_train));
  mu = {mu1(:),mu2(:)};
  x = {fem.mesh.p(1,:)',fem.mesh.p(2,:)'};
  
%% Offline Stage: Perform empirical interpolation
  EIM = performEmpInt(g,x,mu,tol);
  
  figure(1)
  semilogy(1:EIM.Q_max,EIM.eps_star,'r-')
  title('EIM - Maximal Approximation Error during Greedy')
  xlabel Q; 
  ylabel('$\max\limits_{g \in G} \Vert g - I_Q g \Vert_\infty$', 'Interpreter', 'LaTex');
  legend('EIM Error'); 
  
%% Online Stage: Get approximation and error estimator for test set

  Xi_test = -10.^(-2 + 2 .* rand(n_test,2));
  mu = num2cell(Xi_test,1);
  
  X = cellfun(@(x) repmat(x,1,numel(mu{1})),x,'UniformOutput',false);
  Mu = cellfun(@(mu) repmat(mu',numel(x{1}),1),mu,'UniformOutput',false);

  G = g(X{:},Mu{:});

  % check approximation and error estimator
  G_err = zeros(EIM.Q_max,n_test);
  G_errEst = zeros(EIM.Q_max,n_test);
  for Q=1:EIM.Q_max
    [G_coeffs,G_errEst(Q,:)] = getEIMCoeffs(EIM,Q,Q+1,mu{:});
    
    % ====== TODO =======
    G_err(Q,:) = .....
    % ===================
  end

  figure(2)
  semilogy(1:EIM.Q_max,G_err(:,1:3),'-');
  hold on;
  semilogy(1:EIM.Q_max,G_errEst(:,1:3),'--');
  semilogy(1:EIM.Q_max,EIM.eps_star,'k:');
  xlabel('Q');
  ylabel('||g(\mu)-I_Q g(\mu)||_{\infty}','Interpreter','LaTex');
  title('EIM: Approximation error and error estimator for g');

  figure(3)
  G_eff = G_errEst./G_err;
  semilogy(1:EIM.Q_max,[min(G_eff,[],2),mean(G_eff,2),max(G_eff,[],2)]);
  xlabel('Q');
  ylabel('\eta^Q(\mu)');
  legend('min','mean','max');
  title('EIM: Efficiency of error estimator for g');